package com.bomostory.sceneeditmodule.utils

import com.bomostory.sceneeditmodule.basicdata.Scene
import com.bomostory.sceneeditmodule.basicdata.Story
import io.reactivex.Observable
import io.reactivex.Observer
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit
import java.util.concurrent.atomic.AtomicLong

class MoviePlayer {
    interface OnMoviePlayListener {
        fun onMovieSceneUpdate(scene: Scene)
        fun onMoviePlayViewUpdate(x: Int)
        fun onMoviePlayTimeUpdate(time: Long)
        fun onMoviePlayComplete()
    }

    companion object {
        const val INTERVAL_PERIOD: Long = 1
    }

    var story: Story? = null
        set(value) {
            field = value
            value?.apply {
                for (scene in scenes) {
                    scene.record?.period?.let {
                        totalPeriod += it
                    }
                }
            }
        }

    var totalPeriod = 0L

    var musicPlayer = MusicPlayer()

    var moviePlayListener: OnMoviePlayListener? = null
    private val lastTick = AtomicLong(0L)
    private var disposable: Disposable? = null

    fun addMusic(path: String) {
        musicPlayer.add(path)
    }

    fun removeMusic(path: String) {
        musicPlayer.remove(path)
    }

    fun init() {
        disposable?.apply {
            if (!isDisposed) {
                dispose()
            }
        }
        moviePlayListener?.onMoviePlayViewUpdate(0)
        moviePlayListener?.onMoviePlayTimeUpdate(0)
        lastTick.set(0)
        musicPlayer.stop()
    }

    fun play() {
        Observable.interval(INTERVAL_PERIOD, TimeUnit.MILLISECONDS)
                .map {
                    lastTick.getAndIncrement()
                }
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : Observer<Long> {
                    override fun onComplete() {
                        init()
                        moviePlayListener?.onMoviePlayComplete()
                    }

                    override fun onError(e: Throwable) {
                    }

                    override fun onSubscribe(d: Disposable) {
                        disposable = d

                        if (lastTick.get() == 0L) {
                            musicPlayer.play()
                        } else {
                            musicPlayer.start()
                        }
                    }

                    override fun onNext(t: Long) {
                        story?.apply {

                            updateMovie(t)

                            if (t > (totalPeriod / INTERVAL_PERIOD)) {
                                disposable?.dispose()
                                onComplete()
                            }
                        }
                    }
                })
    }

    fun pause() {
        disposable?.dispose()
        musicPlayer.pause()
    }

    fun seekTo(mSec: Int?) {
        mSec?.let {
            lastTick.set(it.toLong())
            updateMovie(mSec.toLong())
            musicPlayer.seekTo(it)
        }
    }

    private fun updateMovie(t: Long) {
        val scene = getScene(t)
        scene?.apply {
            moviePlayListener?.onMovieSceneUpdate(this)
            record?.let {
                for (track in it.tracks) {
                    if (track.time == t) {
                        moviePlayListener?.onMoviePlayViewUpdate(track.positionX)
                    }
                }
            }
        }
        moviePlayListener?.onMoviePlayTimeUpdate(t)
    }

    private fun getScene(t: Long): Scene? {
        var currentScene: Scene? = null
        var startPeriod = 0L
        var endPeriod = 0L

        story?.apply {
            for (scene in scenes) {
                scene.record?.apply {
                    endPeriod += period
                    if (t in startPeriod..endPeriod) {
                        currentScene = scene
                        startPeriod = endPeriod
                    }
                }
            }
        }

        return currentScene
    }
}